//=============================================================================
// DMVehicleFactory.
//   Author:  David M. Pochron (Google my name for current e-mail address)
//   Feel free to modify this to suit your needs, as long as proper credit
//   is given.
//=============================================================================
class DMVehicleFactory extends SVehicleFactory
	placeable;

// Internal vars
var bool bOnPad;				// vehicle last spawned is sitting on the pad and hasn't been used yet
var bool bMaxReached;			// if we reach the max # of vehicles this factory can produce, this gets set to true (so we know when to do an immediate respawn when a vehicle is destroyed)

// Property vars
								// Note: Any of the respawn times can be set to -1 to disable a timed respawn of that type
								// In general, if you are using triggers to spawn vehicles w/ this factory, set all to -1
var() float initialSpawnDelay;	// how long should we wait before spawning the first vehicle for this factory after map opens?
var() float respawnDelay;		// # of seconds after a vehicle on a pad is used before we spawn a new one
var() float maxRespawnDelay;	// if we reach the max # of vehicles, this is how long we wait before doing a spawn after max goes down again

var() bool bWaitForClear;		// if TRUE, factory will wait for the pad to be clear before trying to spawn a vehicle
var() bool bSpawnProtected;		// if TRUE, vehicle is protected from damage until it is first used
var() bool bTeamLocked;			// if TRUE, vehicle is locked except for indicated team below
var() bool bNeutralExit;		// if TRUE, vehicle becomes teamless when a player/bot exits the vehicle
var() byte VehicleTeam;			// team # to assign spawned vehicles to.  Set to 255 (default) to allow anyone to grab a vehicle from this factory
var() int VehicleHealth;		// set vehicles spawned by this factory to this amount of health (0=use default for specified vehicle)


function PreBeginPlay()
{
	local GameInfo info;

	// DMP a bit of code taken from the VehicleOverride class available on the net:
	foreach DynamicActors(class'Engine.Gameinfo', info)		// without this, vehicles won't spawn in DM maps
		info.bAllowVehicles = true;

	Super.PreBeginPlay();
}


simulated event PostBeginPlay()
{
	Super.PostBeginPlay();

	// DMP both ONSVehicleFactory and ASVehicleFactory do this, so I guess we will too....
	if (Level.NetMode != NM_DedicatedServer)
		VehicleClass.static.StaticPrecache(Level);

	bOnPad = false;
	bMaxReached = false;
	if (initialSpawnDelay >= 0)					// mapper can set this to -1 to disable auto respawn for triggered vehicles
		SetTimer(initialSpawnDelay, false);
}


event VehiclePossessed(Vehicle v)
{
	//log("Possessed: prot="$v.bSpawnProtected$"  team="$v.Team);
	v.bSpawnProtected = false;

	if (v.bEnterringUnlocks)					// if this vehicle hasn't been used yet (still virgin on the pad)
	{
		v.bEnterringUnlocks = false;			// clear this to indicate vehicle has been used at least once
		bOnPad = false;							// clear this to indicate area is clear for a new vehicle spawn
		if (respawnDelay >= 0)					// mapper can set this to -1 to disable auto respawn for triggered vehicles
			SetTimer(respawnDelay, false);
	}
}


event VehicleUnPossessed(Vehicle v)
{
	if (bNeutralExit)
		v.Team = 255;			// set abandoned vehicle to no team when player exits it - also disables repairability
}


event VehicleDestroyed(Vehicle v)
{
	//log("Destroyed: prot="$v.bSpawnProtected$"  team="$v.Team);
	if (bMaxReached)			// if this is true, our spawn delay elapsed and we tried to make a new vehicle, but we were at max
	{
		bMaxReached = false;	// so set up a timer to spawn a new vehicle almost immediately
		if (respawnDelay >= 0)	// only do timed respawn if mapper wants us to
			SetTimer(maxRespawnDelay, false);
	}
	else if (v.bEnterringUnlocks)			// someone destroyed the just-spawned vehicle on the pad!
	{
		bOnPad = false;						// clear this to indicate area is clear for a new vehicle spawn
		if (respawnDelay >= 0)				// mapper can set this to -1 to disable auto respawn for triggered vehicles
			SetTimer(respawnDelay, false);	// this is all we can do because we don't save how long ago this vehicle was spawned
	}

	super.VehicleDestroyed(v);
}


function Timer()
{
	local Pawn P;
	local bool bBlocked;
	local Vehicle newVehicle;

	//log("attempting respawn.... vcount="$VehicleCount);

	if (VehicleCount < MaxVehicleCount)
	{
		if (bOnPad == false)		// don't even bother making another if untouched vehicle is still sitting on the pad....
		{
			// Don't spawn a vehicle if someone or something is blocking the pad (if enabled)
			if (bWaitForClear)
			{
				// DMP: taken from class ONSVehicleFactory
				foreach CollidingActors(class'Pawn', P, VehicleClass.default.CollisionRadius * 1.25)
					bBlocked = true;

				if (bBlocked)
				{
					SetTimer(3, false);		// try again later
					return;
				}
			}

			if (VehicleClass != None)
			{
				newVehicle = spawn(VehicleClass,,, Location, Rotation);
				if (newVehicle != None)
				{
					bOnPad = true;							// so we know when player uses them vehicle we can spawn a new one
					VehicleCount++;
					newVehicle.ParentFactory = self;
					newVehicle.Team = VehicleTeam;
					newVehicle.bTeamLocked = bTeamLocked;
					newVehicle.bSpawnProtected = bSpawnProtected;
					newVehicle.bEnterringUnlocks = true;	// All unused vehicles from this factory initially have this set
					if (VehicleHealth > 0)
						newVehicle.Health = VehicleHealth;
				}
			}
			else
				log("WARNING:"@self@"has no vehicle class set!");
		}
	}
	else
		bMaxReached = true;			// we tried to spawn a vehicle, but we hit the max allowed - set flag so we know to spawn immediately
									// when a vehicle is destroyed
}


event Trigger(Actor Other, Pawn EventInstigator)
{
	// DMP: SVehicleFactory does this check but I don't think it's necessary and limits the types of triggers someone may want to use....
	//      Let me know however if my reasoning is in error.
	//if (!EventInstigator.IsA('UnrealPawn'))
	//	return;

	Timer();		// call timer function to make a vehicle spawn
}


/*defaultproperties()
	bWaitForClear = true
	bSpawnProtected = true
	bTeamLocked = false;
	bNeutralExit = true;
	VehicleTeam = 255
	initialSpawnDelay = 1.0
	respawnDelay = 60.0
	maxRespawnDelay = 1.0
	VehicleHealth = 0
*/

defaultproperties
{
     initialSpawnDelay=1.000000
     RespawnDelay=60.000000
     maxRespawnDelay=1.000000
     bWaitForClear=True
     bSpawnProtected=True
     bNeutralExit=True
     VehicleTeam=255
     MaxVehicleCount=5
}
